@@ -7,24 +7,23 @@ if False:  # MYPY
 
 _package_data = dict(
     full_package_name='ruamel.yaml',
-    version_info=(0, 15, 14),
-    __version__='0.15.14',
+    version_info=(0, 15, 19),
+    __version__='0.15.19',
     author='Anthon van der Neut',
     author_email='a.van.der.neut@ruamel.eu',
     description='ruamel.yaml is a YAML parser/emitter that supports roundtrip preservation of comments, seq/map flow style, and map key order',  # NOQA
     entry_points=None,
     since=2014,
-    extras_require={':platform_python_implementation=="CPython" and python_version<="2.7"': [
-        'ruamel.ordereddict',
-        ], 'jinja2': ['ruamel.yaml.jinja2>=0.2']},
+    extras_require={
+        ':platform_python_implementation=="CPython" and python_version<="2.7"':
+        ['ruamel.ordereddict'],
+        'jinja2': ['ruamel.yaml.jinja2>=0.2'],
+    },
     ext_modules=[dict(
-            name='_ruamel_yaml',
+        name='_ruamel_yaml',
             src=['ext/_ruamel_yaml.c', 'ext/api.c', 'ext/writer.c', 'ext/dumper.c',
-                'ext/loader.c',
-                'ext/reader.c',
-                'ext/scanner.c',
-                'ext/parser.c',
-                'ext/emitter.c',
+                 'ext/loader.c', 'ext/reader.c', 'ext/scanner.c', 'ext/parser.c',
+                 'ext/emitter.c',
             ],
             lib=[],
             test='#include "ext/yaml.h"\n\nint main(int argc, char* argv[])\n{\nyaml_parser_t parser;\nparser = parser;  /* prevent warning */\nreturn 0;\n}\n',  # NOQA
@@ -51,6 +50,7 @@ _package_data = dict(
         deps='ruamel.std.pathlib',
         fl8excl='_test/lib',
     ),
+    rtfd=47359,
 )  # type: Dict[Any, Any]
 
 
